/*
 * Filesystem using Amazon S3
 * Author: Avinash Kulkarni
 * NetID: ak678
 * Original shell program credits: Tudor Marian
 */


#include <stdio.h>
#include <stdlib.h>

#include "s3fs.h"
#include "s3ops.h"

static const int multithreaded = 0;


static char inode_bucket[] = "edu-cornell-cs-cs6464-ak678-meta";
static char data_bucket[] = "edu-cornell-cs-cs6464-ak678-data";

static const char *accessKeyIdG = NULL;
static const char *secretAccessKeyG = NULL;

struct s3fs_buckets s3fs_bucket_names = {inode_bucket, data_bucket};

int ll_fuse_main(int argc, char *argv[])
{
	struct fuse_chan *ch;
	struct fuse_lowlevel_ops s3fs_ops;
	char *mountpoint;
	int err = -1;	
	struct fuse_args args = FUSE_ARGS_INIT(argc, argv);

	s3fs_ops_init(&s3fs_ops);

	if (fuse_parse_cmdline(&args, &mountpoint, NULL, NULL) != -1 && (ch
			= fuse_mount(mountpoint, &args)) != NULL)
	{
		struct fuse_session *se;

		se = fuse_lowlevel_new(&args, &s3fs_ops, sizeof(s3fs_ops), 
				&s3fs_bucket_names);
		if (se != NULL)
		{
			if (fuse_set_signal_handlers(se) != -1)
			{
				fuse_session_add_chan(se, ch);
				if (multithreaded)
					err = fuse_session_loop_mt(se);
				else
					err = fuse_session_loop(se);
				fuse_remove_signal_handlers(se);
				fuse_session_remove_chan(ch);
			}
			fuse_session_destroy(se);
		}
		fuse_unmount(mountpoint, ch);
	}
	fuse_opt_free_args(&args);

	return err ? 1 : 0;
}

int main(int argc, char *argv[])
{
	char **fuse_argv;
	int fuse_argc = argc;

	if (!(fuse_argv = (char **)malloc((argc+2) * sizeof(char *))))
	{
		fprintf(stderr, "Malloc failed\n");
		exit(1);
	}
	memcpy(fuse_argv, argv, argc * sizeof(char *));
	
#ifdef __APPLE__
	fuse_argv[fuse_argc++] = "-o";
	fuse_argv[fuse_argc++] = "nolocalcaches"; // no dir entry caching
#endif // __APPLE__

	accessKeyIdG = getenv("S3_ACCESS_KEY_ID");
	if (!accessKeyIdG)
	{
		fprintf(stderr, "Missing environment variable S3_ACCESS_KEY_ID\n");
		return -1;
	}
	secretAccessKeyG = getenv("S3_SECRET_ACCESS_KEY");
	if (!secretAccessKeyG)
	{
		fprintf(stderr, "Missing environment variable S3_SECRET_ACCESS_KEY\n");
		return -1;
	}

	/* make sure you don't forget about this */
	s3_init_credentials(accessKeyIdG, secretAccessKeyG);
	
	/* start by formatting */
	if (!s3fs_format(inode_bucket, data_bucket))
	{
		printf("Unable to format s3fs\n");
		exit(1);
	}
	return ll_fuse_main(fuse_argc, fuse_argv);
}

